==> castor_extractor-0.16.1-py3-none-any.whl: castor_extractor/warehouse/databricks/extract.py <==
import logging
from typing import Dict, Optional

from ...utils import AbstractStorage, LocalStorage, write_summary
from ..abstract import (
    CATALOG_ASSETS,
    EXTERNAL_LINEAGE_ASSETS,
    QUERIES_ASSETS,
    VIEWS_ASSETS,
    SupportedAssets,
    TimeFilter,
    WarehouseAsset,
    WarehouseAssetGroup,
    common_args,
)
from .client import DatabricksClient
from .credentials import to_credentials

DATABRICKS_ASSETS: SupportedAssets = {
    WarehouseAssetGroup.CATALOG: CATALOG_ASSETS,
    WarehouseAssetGroup.QUERY: QUERIES_ASSETS,
    WarehouseAssetGroup.ROLE: (WarehouseAsset.USER,),
    WarehouseAssetGroup.VIEW_DDL: VIEWS_ASSETS,
    WarehouseAssetGroup.EXTERNAL_LINEAGE: EXTERNAL_LINEAGE_ASSETS,
}

logger = logging.getLogger(__name__)

OTimeFilter = Optional[TimeFilter]
Paths = Dict[str, str]


class DatabricksExtractionProcessor:
    """Databricks' API-based extraction management"""

    def __init__(
        self,
        client: DatabricksClient,
        storage: AbstractStorage,
        skip_existing: bool = False,
    ):
        self._client = client
        self._storage = storage
        self._skip_existing = skip_existing

    def _should_not_reextract(self, asset_group) -> bool:
        """helper function to determine whether we need to extract"""
        if not self._skip_existing:
            return False

        for asset in DATABRICKS_ASSETS[asset_group]:
            if not self._storage.exists(asset.value):
                return False

        logger.info(f"Skipped, files for {asset_group.value} already exist")
        return True

    def _existing_group_paths(self, asset_group: WarehouseAssetGroup) -> Paths:
        return {
            a.value: self._storage.path(a.value)
            for a in DATABRICKS_ASSETS[asset_group]
        }

    def extract_catalog(self) -> Paths:
        """
        Extract all catalog assets (from database to column)
        and return the locations of the extracted data
        """
        if self._should_not_reextract(WarehouseAssetGroup.CATALOG):
            return self._existing_group_paths(WarehouseAssetGroup.CATALOG)

        catalog_locations: Dict[str, str] = dict()
        databases = self._client.databases()
        location = self._storage.put(WarehouseAsset.DATABASE.value, databases)
        catalog_locations[WarehouseAsset.DATABASE.value] = location
        logger.info(f"Extracted {len(databases)} databases to {location}")

        schemas = self._client.schemas(databases)
        location = self._storage.put(WarehouseAsset.SCHEMA.value, schemas)
        catalog_locations[WarehouseAsset.SCHEMA.value] = location
        logger.info(f"Extracted {len(schemas)} schemas to {location}")

        del databases

        tables, columns = self._client.tables_and_columns(schemas)

        location = self._storage.put(WarehouseAsset.TABLE.value, tables)
        catalog_locations[WarehouseAsset.TABLE.value] = location
        logger.info(f"Extracted {len(tables)} tables to {location}")

        location = self._storage.put(WarehouseAsset.COLUMN.value, columns)
        catalog_locations[WarehouseAsset.COLUMN.value] = location
        logger.info(f"Extracted {len(columns)} columns to {location}")
        return catalog_locations

    def extract_query(self, time_filter: OTimeFilter = None) -> Paths:
        """extract yesterday's queries and return their location"""
        if self._should_not_reextract(WarehouseAssetGroup.QUERY):
            return self._existing_group_paths(WarehouseAssetGroup.QUERY)

        queries = self._client.queries(time_filter)
        location = self._storage.put(WarehouseAsset.QUERY.value, queries)
        logger.info(f"Extracted {len(queries)} queries to {location}")
        return {WarehouseAsset.QUERY.value: location}

    def extract_role(self) -> Paths:
        """extract roles (users) and return their location"""
        if self._should_not_reextract(WarehouseAssetGroup.ROLE):
            return self._existing_group_paths(WarehouseAssetGroup.ROLE)

        users = self._client.users()
        location = self._storage.put(WarehouseAsset.USER.value, users)
        logger.info(f"Extracted {len(users)} users to {location}")
        return {WarehouseAsset.USER.value: location}

    def extract_view_ddl(self) -> Paths:
        """extract view ddl (using the same route as tables)"""
        if self._should_not_reextract(WarehouseAssetGroup.VIEW_DDL):
            return self._existing_group_paths(WarehouseAssetGroup.VIEW_DDL)

        databases = self._client.databases()
        schemas = self._client.schemas(databases)
        view_ddl = self._client.view_ddl(schemas)
        location = self._storage.put(WarehouseAsset.VIEW_DDL.value, view_ddl)
        logger.info(f"Extracted {len(view_ddl)} view_ddl to {location}")
        return {WarehouseAsset.VIEW_DDL.value: location}


def extract_all(**kwargs) -> None:
    """
    Extract all assets from Databricks and store the results in CSV files
    Time filter scope for `Queries` = the day before (from 12AM to 12AM)
    """
    output_directory, skip_existing = common_args(kwargs)

    client = DatabricksClient(
        credentials=to_credentials(kwargs),
        db_allowed=kwargs.get("db_allowed"),
        db_blocked=kwargs.get("db_blocked"),
    )

    storage = LocalStorage(directory=output_directory)

    extractor = DatabricksExtractionProcessor(
        client=client,
        storage=storage,
        skip_existing=skip_existing,
    )

    extractor.extract_catalog()
    extractor.extract_query()
    extractor.extract_role()
    extractor.extract_view_ddl()

    write_summary(
        output_directory,
        storage.stored_at_ts,
        client_name=client.name(),
    )

==> castor_extractor-0.16.3-py3-none-any.whl: castor_extractor/warehouse/databricks/extract.py <==
import logging
from typing import Dict, Optional

from ...utils import AbstractStorage, LocalStorage, write_summary
from ..abstract import (
    CATALOG_ASSETS,
    EXTERNAL_LINEAGE_ASSETS,
    QUERIES_ASSETS,
    VIEWS_ASSETS,
    SupportedAssets,
    TimeFilter,
    WarehouseAsset,
    WarehouseAssetGroup,
    common_args,
)
from .client import DatabricksClient
from .credentials import to_credentials

DATABRICKS_ASSETS: SupportedAssets = {
    WarehouseAssetGroup.CATALOG: CATALOG_ASSETS,
    WarehouseAssetGroup.QUERY: QUERIES_ASSETS,
    WarehouseAssetGroup.ROLE: (WarehouseAsset.USER,),
    WarehouseAssetGroup.VIEW_DDL: VIEWS_ASSETS,
    WarehouseAssetGroup.EXTERNAL_LINEAGE: EXTERNAL_LINEAGE_ASSETS,
}

logger = logging.getLogger(__name__)

OTimeFilter = Optional[TimeFilter]
Paths = Dict[str, str]


class DatabricksExtractionProcessor:
    """Databricks' API-based extraction management"""

    def __init__(
        self,
        client: DatabricksClient,
        storage: AbstractStorage,
        skip_existing: bool = False,
    ):
        self._client = client
        self._storage = storage
        self._skip_existing = skip_existing

    def _should_not_reextract(self, asset_group) -> bool:
        """helper function to determine whether we need to extract"""
        if not self._skip_existing:
            return False

        for asset in DATABRICKS_ASSETS[asset_group]:
            if not self._storage.exists(asset.value):
                return False

        logger.info(f"Skipped, files for {asset_group.value} already exist")
        return True

    def _existing_group_paths(self, asset_group: WarehouseAssetGroup) -> Paths:
        return {
            a.value: self._storage.path(a.value)
            for a in DATABRICKS_ASSETS[asset_group]
        }

    def extract_catalog(self) -> Paths:
        """
        Extract all catalog assets (from database to column)
        and return the locations of the extracted data
        """
        if self._should_not_reextract(WarehouseAssetGroup.CATALOG):
            return self._existing_group_paths(WarehouseAssetGroup.CATALOG)

        catalog_locations: Dict[str, str] = dict()
        databases = self._client.databases()
        location = self._storage.put(WarehouseAsset.DATABASE.value, databases)
        catalog_locations[WarehouseAsset.DATABASE.value] = location
        logger.info(f"Extracted {len(databases)} databases to {location}")

        schemas = self._client.schemas(databases)
        location = self._storage.put(WarehouseAsset.SCHEMA.value, schemas)
        catalog_locations[WarehouseAsset.SCHEMA.value] = location
        logger.info(f"Extracted {len(schemas)} schemas to {location}")

        del databases

        users = self._client.users()
        tables, columns = self._client.tables_and_columns(schemas, users)

        location = self._storage.put(WarehouseAsset.TABLE.value, tables)
        catalog_locations[WarehouseAsset.TABLE.value] = location
        logger.info(f"Extracted {len(tables)} tables to {location}")

        location = self._storage.put(WarehouseAsset.COLUMN.value, columns)
        catalog_locations[WarehouseAsset.COLUMN.value] = location
        logger.info(f"Extracted {len(columns)} columns to {location}")
        return catalog_locations

    def extract_query(self, time_filter: OTimeFilter = None) -> Paths:
        """extract yesterday's queries and return their location"""
        if self._should_not_reextract(WarehouseAssetGroup.QUERY):
            return self._existing_group_paths(WarehouseAssetGroup.QUERY)

        queries = self._client.queries(time_filter)
        location = self._storage.put(WarehouseAsset.QUERY.value, queries)
        logger.info(f"Extracted {len(queries)} queries to {location}")
        return {WarehouseAsset.QUERY.value: location}

    def extract_role(self) -> Paths:
        """extract roles (users) and return their location"""
        if self._should_not_reextract(WarehouseAssetGroup.ROLE):
            return self._existing_group_paths(WarehouseAssetGroup.ROLE)

        users = self._client.users()
        location = self._storage.put(WarehouseAsset.USER.value, users)
        logger.info(f"Extracted {len(users)} users to {location}")
        return {WarehouseAsset.USER.value: location}

    def extract_view_ddl(self) -> Paths:
        """extract view ddl (using the same route as tables)"""
        if self._should_not_reextract(WarehouseAssetGroup.VIEW_DDL):
            return self._existing_group_paths(WarehouseAssetGroup.VIEW_DDL)

        databases = self._client.databases()
        schemas = self._client.schemas(databases)
        view_ddl = self._client.view_ddl(schemas)
        location = self._storage.put(WarehouseAsset.VIEW_DDL.value, view_ddl)
        logger.info(f"Extracted {len(view_ddl)} view_ddl to {location}")
        return {WarehouseAsset.VIEW_DDL.value: location}


def extract_all(**kwargs) -> None:
    """
    Extract all assets from Databricks and store the results in CSV files
    Time filter scope for `Queries` = the day before (from 12AM to 12AM)
    """
    output_directory, skip_existing = common_args(kwargs)

    client = DatabricksClient(
        credentials=to_credentials(kwargs),
        db_allowed=kwargs.get("db_allowed"),
        db_blocked=kwargs.get("db_blocked"),
    )

    storage = LocalStorage(directory=output_directory)

    extractor = DatabricksExtractionProcessor(
        client=client,
        storage=storage,
        skip_existing=skip_existing,
    )

    extractor.extract_catalog()
    extractor.extract_query()
    extractor.extract_role()
    extractor.extract_view_ddl()

    write_summary(
        output_directory,
        storage.stored_at_ts,
        client_name=client.name(),
    )
